C This subroutine was prepared by S. Shafer, C. Hu, for NONMEM use with 1, 2
C and 3-compartment models parameterized in volumes and clearances using multiple
C or single infusions as input. It reads *.drg files and covariate files that
C are compatable with those used for INF3, a MKMODEL pharmacokinetic model

C The programs loads a covariates file, called COVAR.NM. The structure of the
C COVAR.NM file is first, the DRG file for the nth subject, followed by the
C individual covariates.
C
C The program then loads the file ZP.NM.  The interpretation of the ZP
C table, a 6 x 11 array, is as follows:

C   if ZP(line, 11) = 0 (first 3 terms combined by addition)
C       parameter (line) =  [(THETA(ZP(line,1))*COV(ZP(line,2))^THETA(ZP(line,3)) +
C                           THETA(ZP(line,4))*COV(ZP(line,5))^THETA(ZP(line,6)) +
C                           THETA(ZP(line, 7))^COV(ZP(line,8)))] *
C                           THETA(ZP(line, 9))^COV(ZP(line,10))
C
C   if ZP(line, 11) = 1 (first 3 terms combined by multiplication)
C       parameter (line) =  [(THETA(ZP(line,1))*COV(ZP(line,2))^THETA(ZP(line,3)) *
C                           THETA(ZP(line,4))*COV(ZP(line,5))^THETA(ZP(line,6)) *
C                           THETA(ZP(line, 7))^COV(ZP(line,8)))] +
C                           THETA(ZP(line, 9))^COV(ZP(line,10))

C   Where "line" ranges from 1 to 6, depending on the parameter:
C       V1 = parameter(1)
C       V2 = parameter(2)
C       V3 = parameter(3)
C       CL1 = parameter(4)
C       CL2 = parameter(5)
C       CL3 = parameter(6)

C   ETA values are also assigned using a table, so that specific ETA values
C   may be omitted if not supported by the data.  They are loaded from the table
C   "ZPETA.NM" into a 6 x 1 array.  The array correlates with the volume and
C   clearance parameters as shown above.  If the ZPETA table has a nonzero value
C   then an ETA value will be assigned to the individual parameter.  If the value
C   is zero, then no ETA is assigned.

C   The model collapses to a 2 compartment if the third and sixth lines are of the ZP table
C   are filled with zeros, and to a one compartment model if the second and fifth lines
C   of the ZP table are filles with zeros.  The corresponding lines of the ZPETA table are
C   automatically set to 0 when the model collapses to a two or one compartment model.

      SUBROUTINE PRED (ICALL,NEWIND,THETA,DATREC,INDXS,F,G,H)
      IMPLICIT DOUBLE PRECISION (A-Z)
      REAL DATREC
      SAVE
      INTEGER ICALL,NEWIND,INDXS
      LOGICAL CHKVAL
      DIMENSION THETA(*),DATREC(*),INDXS(*),G(10,*),H(10,*)
      DIMENSION ETA(10),EPS(10)
      COMMON/NMPRD1/IERPRD,NETEXT
      COMMON/NMPRD2/ETEXT(3)
      COMMON/NMPRD4/V1,V2,V3,CL1,CL2,CL3,TY
      INTEGER IERPRD,NETEXT
      CHARACTER*132 ETEXT
      COMMON/ROCM12/MSEC
      INTEGER MSEC,I,J
      CHARACTER*80 DRGFIL(100)
      REAL COV
      DIMENSION COV(10,100)
      DIMENSION COM(400)
      EQUIVALENCE (V1,COM(1))
      INTEGER DRGNO(100), DRGSTR(100), DRGNUM
      REAL DRGON(100001),DRGOFF(100001),DRGRAT(100001)
      COMMON/VCL1/RATE,TOF,TON,CTIME,
     $  L1,DL1E1,DL1E2,DL1E3,DL1E4,DL1E5,DL1E6,
     $  L2,DL2E1,DL2E2,DL2E3,DL2E4,DL2E5,DL2E6,
     $  L3,DL3E1,DL3E2,DL3E3,DL3E4,DL3E5,DL3E6,
     $  C1,DC1E1,DC1E2,DC1E3,DC1E4,DC1E5,DC1E6,
     $  C2,DC2E1,DC2E2,DC2E3,DC2E4,DC2E5,DC2E6,
     $  C3,DC3E1,DC3E2,DC3E3,DC3E4,DC3E5,DC3E6,
     $  S1P,DS1PE1,DS1PE2,DS1PE3,DS1PE4,DS1PE5,DS1PE6,
     $  S2P,DS2PE1,DS2PE2,DS2PE3,DS2PE4,DS2PE5,DS2PE6,
     $  S3P,DS3PE1,DS3PE2,DS3PE3,DS3PE4,DS3PE5,DS3PE6
      INTEGER FLAG,ZP(6,11),ZPETA(6),MODEL,ID,LSTETA
      CHARACTER*10 ZPTEXT(6)
      CHARACTER*81 RECORD
      INTEGER DRGFLG,POS,POS1,RECNO,CHRVAL,SGNFLG,MAXREC,DRGLIN
      CHARACTER CHAR
      DATA (ZPTEXT(I),I=1,6)/'V1','V2','V3','CL1','CL2','CL3'/

C BEGIN ROUTINE HERE
C======================================================================
C First Pass Initialization
      IF (FLAG.EQ.0) THEN
         FLAG = 1
         WRITE(*,*)'Opening ZP.NM'
         WRITE(6,*)'Structure of ZP.NM:'
         OPEN (21,FILE='ZP.NM', FORM='FORMATTED', STATUS='OLD',
     $      ERR=9000)
         REWIND 21
         DO 10 I=1,6
             READ(21,fmt=*,err=9001,end=9002) (ZP(I,J), J=1,11)
             WRITE(*,fmt=11)ZPTEXT(I), (ZP(I,J), J=1,11)
10           WRITE(6,fmt=11)ZPTEXT(I), (ZP(I,J), J=1,11)
11       FORMAT(' ',A6,': ',2(3(I2,' '),'  '),2(2(I2,' '),'  '),' ',I2)
         CLOSE(21)

         WRITE(*,*)'Opening ZPETA.NM'
         WRITE(6,*)'Structure of ZPETA.NM'
         OPEN (21,FILE='ZPETA.NM', FORM='FORMATTED', STATUS='OLD',
     $      ERR=9003)
         REWIND 21
         J=1
         DO 20 I=1,6
             READ(21,fmt=*,err=9004,end=9005) ZPETA(I)
             WRITE(*,*) ZPETA(I)
             WRITE(6,*) ZPETA(I)
C            Transform ZPETA into an array of pointers
             IF (ZPETA(I).EQ.0) THEN
                ZPETA(I) = 8
             ELSE
                ZPETA(I) = J
                J=J+1
             ENDIF
20       CONTINUE
         LSTETA=J
         CLOSE(21)
         IF (ZP(2,1)+ZP(2,2)+ZP(2,3)+ZP(2,4)+ZP(2,5)+
     $       ZP(2,6)+ZP(2,7)+ZP(2,8)+ZP(2,9)+ZP(2,10).EQ.0) THEN
               MODEL=1
               WRITE(*,*) "One Compartment Model"
               WRITE(6,*) "One Compartment Model"
               IF(ZPETA(2)+ZPETA(3)+ZPETA(5)+ZPETA(6).NE.28) THEN
                   WRITE(*,*)'ZPETA Elements 2,3,5,6 set to 0'
                   WRITE(6,*)'ZPETA Elements 2,3,5,6 set to 0'
                   ZPETA(2)=8
                   ZPETA(3)=8
                   ZPETA(5)=8
                   ZPETA(6)=8
               ENDIF
         ELSE
              IF (ZP(3,1)+ZP(3,2)+ZP(3,3)+ZP(3,4)+ZP(3,5)+
     $            ZP(3,6)+ZP(3,7)+ZP(3,8)+ZP(3,9)+ZP(3,10).EQ.0) THEN
                    MODEL=2
                    WRITE(*,*) "Two Compartment Model"
                    WRITE(6,*) "Two Compartment Model"
                    IF (ZPETA(3)+ZPETA(6).NE.14) THEN
                        WRITE(*,*)'ZPETA Elements 3 and 6 set to 0'
                        WRITE(6,*)'ZPETA Elements 3 and 6 set to 0'
                        ZPETA(3)=8
                        ZPETA(6)=8
                    ENDIF
              ELSE
                    MODEL=3
                    WRITE(*,*) "Three Compartment Model"
                    WRITE(6,*) "Three Compartment Model"
              ENDIF
         ENDIF

C Modification to run under Windows (Pipes not supported)
         RECNO=0
         OPEN (21,FILE='COVAR.NM',ERR=9006)
         REWIND 21
C        Loop to read in the records
100      RECORD=' '
         READ (21,130,END=200,ERR=120) RECORD
120      CONTINUE
130      FORMAT (A80)
         IF (RECORD.EQ.' ') GO TO 100
C        Output entire record
         WRITE (*,132) RECORD
         WRITE (6,130) RECORD
132      FORMAT('1',A79)
C
         DRGFLG = 0
         POS = 0
         POS1 = 1
         RECNO = RECNO+1
         DRGFIL(RECNO) = ' '
C        Index past white space, if any
133      IF (POS.GT.80) GOTO 100
         POS=POS+1
         CHAR = RECORD(POS:POS)
         IF (CHAR.LT.'0'.OR.CHAR.GT.'9') GOTO 133

C        Now read subject number
         DRGNUM=0
         POS=POS-1
134      POS=POS+1
         IF (POS.GT.80) GOTO 100
         CHAR=RECORD(POS:POS)
         IF (CHAR.LT.'0'.OR.CHAR.GT.'9') GOTO 135
         READ(CHAR,'(I1)') CHRVAL
         DRGNUM=DRGNUM * 10 + CHRVAL
         GOTO 134

C        Move past white space
135      IF (CHAR.LE.' '.OR.CHAR.EQ.',') THEN
             POS=POS+1
             CHAR=RECORD(POS:POS)
             GOTO 135
         END IF

C        Isolate the DRG file
140      IF (DRGFLG.EQ.3) GOTO 150
         CHAR=RECORD(POS:POS)
         DRGFIL(RECNO)(POS1:POS1)=CHAR
         POS1=POS1+1

C        FIND DRG extension
         IF (DRGFLG.EQ.0) THEN
            IF(CHAR.EQ.'d'.OR.CHAR.EQ.'D') THEN
                DRGFLG = 1
            END IF
         ELSE
            IF (DRGFLG.EQ.1) THEN
                IF(CHAR.EQ.'r'.OR.CHAR.EQ.'R') THEN
                    DRGFLG = 2
                ELSE
                    DRGFLG = 0
                ENDIF
            ELSE
                IF(CHAR.EQ.'g'.or.CHAR.EQ.'G') THEN
                    DRGFLG = 3
                ELSE
                    DRGFLG = 0
                ENDIF
            ENDIF
         ENDIF
         POS=POS+1
         IF (POS.GT.80) GOTO 100
         GOTO 140

150      CONTINUE
         DRGNO(RECNO) = DRGNUM
         COVNUM=1

C        Find individual covariates
C        Index to not white space
160      IF (POS.GT.80) GOTO 100
         POS=POS+1
         CHAR = RECORD(POS:POS)
         IF (CHAR.NE.'.'.AND.CHAR.NE.'-'.AND.CHAR.NE.'+'.AND.
     $   (CHAR.LT.'0'.OR.CHAR.GT.'9')) GOTO 160

C        Now read number
         NUM=0
         DECFLG = 0
         SGNFLG = 1
         DECIM = 10
         POS=POS-1
170      POS=POS+1
         IF (POS.GT.80) GOTO 100
         CHAR=RECORD(POS:POS)
         IF (CHAR.NE.'.'.AND.CHAR.NE.'-'.AND.CHAR.NE.'+'.AND.
     $      (CHAR.LT.'0'.OR.CHAR.GT.'9')) GOTO 180
         IF (CHAR.EQ.'.') THEN
            DECFLG=1
            GOTO 170
         ENDIF
         IF (CHAR.EQ.'-') THEN
            SGNFLG=-1
            GOTO 170
         ENDIF
         IF (CHAR.EQ.'+') THEN
            SGNFLG=1
            GOTO 170
         ENDIF
         READ(CHAR,'(I1)') CHRVAL
         IF (DECFLG.EQ.0) THEN
             NUM=NUM * 10 + CHRVAL
         ELSE
             NUM=NUM+CHRVAL/DECIM
             DECIM=DECIM*10
         ENDIF
         GOTO 170

180      COV(COVNUM,RECNO) = NUM * SGNFLG
         COVNUM = COVNUM + 1
         GOTO 160

200      CONTINUE
         MAXREC=RECNO
         CLOSE(21)

C        Now read in all the DRG files
         DRGLIN=1
         DO 250, RECNO=1, MAXREC
         DRGSTR(RECNO)=DRGLIN
         OPEN (21,FILE=DRGFIL(RECNO), FORM='FORMATTED', STATUS='OLD',
     $         ERR=9007)
         REWIND 21
C        Read bolus line
         READ(21,FMT=*,ERR=9008,END=9009) TIMEON, RATE
         IF (TIMEON.NE.9999) THEN
             WRITE(*,*) 'Boluses not allowed in NMVCL'
             STOP
         ENDIF
220      READ(21,FMT=*,ERR=9008,END=9009)
     $      DRGON(DRGLIN), DRGOFF(DRGLIN), DRGRAT(DRGLIN)
         DRGLIN=DRGLIN+1
         IF (DRGLIN.GT.100000) THEN
             WRITE(*,*) 'Total DRG lines exceeds 100,000'
             STOP
         ENDIF

         IF(DRGON(DRGLIN-1).EQ.9999.OR.DRGON(DRGLIN-1).EQ.99999)GOTO 240
         GOTO 220
240      CLOSE (21)
250      CONTINUE

      ENDIF
C=========================================================================
C Get ETA,EPS if needed
      IF (ICALL.EQ.4) THEN
          IF (NEWIND.NE.2) CALL SIMETA(ETA)
          CALL SIMEPS(EPS)
          ETA(8) = 0.D0
      ELSE
          IF (NEWIND.NE.2) THEN
              CALL GETETA(ETA)
              EPS(01) = 0.D0
              ETA(8) = 0.D0
          ENDIF
      ENDIF
      TIME=DATREC(02)

C==========================================================================
C Initialize state variables, exponentials, coefficients and partials,
C if new individual.  This if statement is terminated just prior to the
C double equal bars.
      IF (NEWIND.NE.2) THEN
C        Compute true values, using ZP table
C        and apply ETA, if indicated by ZPETA table

C        Find subject in the covariate table
         ID = DATREC(1)
         DO 300 I=1, MAXREC
            IF (DRGNO(I).EQ.ID) THEN
               RECNO = I
               GOTO 310
            ENDIF
300      CONTINUE
         WRITE (*,*) 'Subject not found in the covariate table:', ID
         STOP

310      V1TV=PRMSET(1,THETA,ZP,COV, RECNO)
         IF (V1TV.LE.0) THEN
             WRITE(*,*)'V1=0'
             WRITE(6,*)'V1=0'
             IERPRD=1
             RETURN
         ENDIF
         E1=ETA(ZPETA(1))
C        Protection against Eta
         IF (E1.GT.10.0D0) THEN
             WRITE(*,*)'ETA on V1 too big'
             IERPRD=1
             RETURN
         ENDIF
         IF (E1.LT.-10.0D0) THEN
             WRITE(*,*)'ETA on V1 too small'
             IERPRD=1
             RETURN
         ENDIF
         V1=V1TV*DEXP(E1)

         IF (MODEL.GT.1) THEN
             V2TV=PRMSET(2,THETA,ZP,COV, RECNO)
             IF (V2TV.LE.0) THEN
                 WRITE(*,*)'V2=0'
                 WRITE(6,*)'V2=0'
                 IERPRD=1
                 RETURN
             ENDIF
             E2=ETA(ZPETA(2))
C            Protection against Eta
             IF (E2.GT.10.0D0) THEN
                 WRITE(*,*)'ETA on V2 too big'
                 IERPRD=1
                 RETURN
             ENDIF
             IF (E2.LT.-10.0D0) THEN
                 WRITE(*,*)'ETA on V2 too small'
                 IERPRD=1
                 RETURN
             ENDIF
             V2=V2TV*DEXP(E2)

             IF (MODEL.GT.2) THEN
                 V3TV=PRMSET(3,THETA,ZP,COV, RECNO)
                 IF (V3TV.LE.0) THEN
                     WRITE(*,*)'V3=0'
                     WRITE(6,*)'V3=0'
                     IERPRD=1
                     RETURN
                 ENDIF
                 E3=ETA(ZPETA(3))
C                Protection against Eta
                 IF (E3.GT.10.0D0) THEN
                     WRITE(*,*)'ETA on V3 too big'
                     IERPRD=1
                     RETURN
                 ENDIF
                 IF (E3.LT.-10.0D0) THEN
                     WRITE(*,*)'ETA on V3 too small'
                     IERPRD=1
                     RETURN
                 ENDIF
                 V3=V3TV*DEXP(E3)
             ENDIF
         ENDIF

         CL1TV=PRMSET(4,THETA,ZP,COV, RECNO)
         IF (CL1TV.LE.0) THEN
             WRITE(*,*)'CL1=0'
             WRITE(6,*)'CL1=0'
             IERPRD=1
             RETURN
         ENDIF
         E4=ETA(ZPETA(4))
C        Protection against Eta
         IF (E4.GT.10.0D0) THEN
             WRITE(*,*)'ETA on CL1 too big',E4, CL1TV
             IERPRD=1
             RETURN
         ENDIF
         IF (E4.LT.-10.0D0) THEN
             WRITE(*,*)'ETA on CL1 too small'
             IERPRD=1
             RETURN
         ENDIF
         CL1=CL1TV*DEXP(E4)

         IF (MODEL.GT.1) THEN
             CL2TV=PRMSET(5,THETA,ZP,COV, RECNO)
             IF (CL2TV.LE.0) THEN
                 WRITE(*,*)'CL2=0'
                 WRITE(6,*)'CL2=0'
                 IERPRD=1
                 RETURN
             ENDIF
             E5=ETA(ZPETA(5))
C            Protection against Eta
             IF (E5.GT.10.0D0) THEN
                 WRITE(*,*)'ETA on CL2 too big'
                 IERPRD=1
                 RETURN
             ENDIF
             IF (E5.LT.-10.0D0) THEN
                 WRITE(*,*)'ETA on CL2 too small'
                 IERPRD=1
                 RETURN
             ENDIF
             CL2=CL2TV*DEXP(E5)

             IF (MODEL.GT.2) THEN
                CL3TV=PRMSET(6,THETA,ZP,COV, RECNO)
                IF (CL3TV.LE.0) THEN
                     WRITE(*,*)'CL3=0'
                     WRITE(6,*)'CL3=0'
                    IERPRD=1
                    RETURN
                ENDIF
                E6=ETA(ZPETA(6))
C               Protection against Eta
                IF (E6.GT.10.0D0) THEN
                    WRITE(*,*)'ETA on CL3 too big'
                    IERPRD=1
                    RETURN
                ENDIF
                IF (E6.LT.-10.0D0) THEN
                    WRITE(*,*)'ETA on CL3 too small'
                    IERPRD=1
                    RETURN
                ENDIF
                CL3=CL3TV*DEXP(E6)
             ENDIF
         ENDIF
C=====================================================================
C Determine rate constants and partials
         DV1E1 = DEXP(E1)*V1TV
         DV1E2 = 0.0D0
         DV1E3 = 0.0D0
         DV1E4 = 0.0D0
         DV1E5 = 0.0D0
         DV1E6 = 0.0D0

         K10 = Cl1/V1

         DK10E1 = -(DEXP(-E1+E4)*CL1TV/V1TV)
         DK10E2 = 0.D0
         DK10E3 = 0.D0
         DK10E4 = DEXP(-E1+E4)*CL1TV/V1TV
         DK10E5 = 0.D0
         DK10E6 = 0.D0

         IF (MODEL.GT.1) THEN
             K12 = Cl2/V1

             DK12E1 = -(DEXP(-E1+E5)*CL2TV/V1TV)
             DK12E2 = 0.D0
             DK12E3 = 0.D0
             DK12E4 = 0.D0
             DK12E5 = DEXP(-E1+E5)*CL2TV/V1TV
             DK12E6 = 0.D0

             K21 = Cl2/V2

             DK21E1 = 0.D0
             DK21E2 = -(DEXP(-E2+E5)*CL2TV/V2TV)
             DK21E3 = 0.D0
             DK21E4 = 0.D0
             DK21E5 = DEXP(-E2+E5)*CL2TV/V2TV
             DK21E6 = 0.D0

             IF (MODEL.GT.2) THEN
                 K13 = Cl3/V1

                 DK13E1 = -(DEXP(-E1+E6)*CL3TV/V1TV)
                 DK13E2 = 0.D0
                 DK13E3 = 0.D0
                 DK13E4 = 0.D0
                 DK13E5 = 0.D0
                 DK13E6 = DEXP(-E1+E6)*CL3TV/V1TV

                 K31 = Cl3/V3

                 DK31E1 = 0.D0
                 DK31E2 = 0.D0
                 DK31E3 = -(DEXP(-E3+E6)*CL3TV/V3TV)
                 DK31E4 = 0.D0
                 DK31E5 = 0.D0
                 DK31E6 = DEXP(-E3+E6)*CL3TV/V3TV
             ENDIF
         ENDIF

C============================================================================
C Determine exponents, coefficients, and partials
C In this section, indention is not used because the Fortran line
C length becomes too much of a hassle (aarghhh!)
         IF (MODEL.EQ.1) THEN

C        One compartment model...............................................

         L1=K10

         DL1E1=DK10E1
         DL1E2=0.D0
         DL1E3=0.D0
         DL1E4=DK10E4
         DL1E5=0.D0
         DL1E6=0.D0

         C1=1/V1
         DC1E1=-1/V1
         DC1E2=0.D0
         DC1E3=0.D0
         DC1E4=0.D0
         DC1E5=0.D0
         DC1E6=0.D0

C        End of one compartment model........................................

         ELSE
         IF (MODEL.EQ.2) THEN

C        Two Compartment model...............................................

         P=K10*K21

         DPE1=K21*DK10E1+K10*DK21E1
         DPE2=K21*DK10E2+K10*DK21E2
         DPE4=K21*DK10E4+K10*DK21E4
         DPE5=K21*DK10E5+K10*DK21E5

         Q=K10+K12+K21

         DQE1=DK10E1+DK12E1+DK21E1
         DQE2=DK10E2+DK12E2+DK21E2
         DQE4=DK10E4+DK12E4+DK21E4
         DQE5=DK10E5+DK12E5+DK21E5

         IF (4.0D0*P.EQ.Q*Q) THEN
               WRITE(*,*) 'Small denominator found'
               IERPRD=1
               RETURN
         ENDIF

         L1=-(-Q-DSQRT(Q*Q-4.0D0*P))/2.0D0

         DL1E1=-(-DQE1-(-4.0D0*DPE1+2.0D0*Q*DQE1)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL1E2=-(-DQE2-(-4.0D0*DPE2+2.0D0*Q*DQE2)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         D1E3=0.D0
         DL1E4=-(-DQE4-(-4.0D0*DPE4+2.0D0*Q*DQE4)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL1E5=-(-DQE5-(-4.0D0*DPE5+2.0D0*Q*DQE5)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL1E6=0.D0

         L2=-(-Q+DSQRT(Q*Q-4.0D0*P))/2.0D0

         DL2E1=-(-DQE1+(-4.0D0*DPE1+2.0D0*Q*DQE1)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL2E2=-(-DQE2+(-4.0D0*DPE2+2.0D0*Q*DQE2)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL2E3=0.D0
         DL2E4=-(-DQE4+(-4.0D0*DPE4+2.0D0*Q*DQE4)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL2E5=-(-DQE5+(-4.0D0*DPE5+2.0D0*Q*DQE5)/
     $         (2*DSQRT(-4.0D0*P+Q*Q)))/2.0D0
         DL2E6=0.D0

C        Check for illegal values that will cause division by 0 or an overflow
         IF (CHKVAL(K21,L1).OR.CHKVAL(K21,L2).CHKVAL(L1,L2)) THEN
               WRITE(*,*) 'Small denominator found'
               IERPRD=1
               RETURN
         ENDIF

         C1=1/V1*(K21-L1)/(L2-L1)

         DC1E1=(DK21E1-DL1E1)/((L2-L1)*V1)-(K21-L1)*(-DL1E1+DL2E1)/
     $         ((L2-L1)*(L2-L1)*V1)-(K21-L1)*DV1E1/((L2-L1)*V1*V1)
         DC1E2=(DK21E2-DL1E2)/((L2-L1)*V1)-(K21-L1)*(-DL1E2+DL2E2)/
     $         ((L2-L1)*(L2-L1)*V1)-(K21-L1)*DV1E2/((L2-L1)*V1*V1)
         DC1E3=0.D0
         DC1E4=(DK21E4-DL1E4)/((L2-L1)*V1)-(K21-L1)*(-DL1E4+DL2E4)/
     $         ((L2-L1)*(L2-L1)*V1)-(K21-L1)*DV1E4/((L2-L1)*V1*V1)
         DC1E5=(DK21E5-DL1E5)/((L2-L1)*V1)-(K21-L1)*(-DL1E5+DL2E5)/
     $         ((L2-L1)*(L2-L1)*V1)-(K21-L1)*DV1E5/((L2-L1)*V1*V1)
         DC1E6=0.D0

         C2=1/V1*(K21-L2)/(L1-L2)

         DC2E1=(DK21E1-DL2E1)/((L1-L2)*V1)-(K21-L2)*(DL1E1-DL2E1)/
     $         ((L1-L2)*(L1-L2)*V1)-(K21-L2)*DV1E1/((L1-L2)*V1*V1)
         DC2E2=(DK21E2-DL2E2)/((L1-L2)*V1)-(K21-L2)*(DL1E2-DL2E2)/
     $         ((L1-L2)*(L1-L2)*V1)-(K21-L2)*DV1E2/((L1-L2)*V1*V1)
         DC2E3=0.D0
         DC2E4=(DK21E4-DL2E4)/((L1-L2)*V1)-(K21-L2)*(DL1E4-DL2E4)/
     $         ((L1-L2)*(L1-L2)*V1)-(K21-L2)*DV1E4/((L1-L2)*V1*V1)
         DC2E5=(DK21E5-DL2E5)/((L1-L2)*V1)-(K21-L2)*(DL1E5-DL2E5)/
     $         ((L1-L2)*(L1-L2)*V1)-(K21-L2)*DV1E5/((L1-L2)*V1*V1)
         DC2E6=0.D0

C        End of two compartment model........................................
         ELSE

C        Three compartment model.............................................
C        Uses method of solution to the cubic equation, complete with partials

         PI=DASIN(1.0D0)*2.0D0

         A0 = K10*K21*K31

         DA0E1=K21*K31*DK10E1+K10*K31*DK21E1+K10*K21*DK31E1
         DA0E2=K21*K31*DK10E2+K10*K31*DK21E2+K10*K21*DK31E2
         DA0E3=K21*K31*DK10E3+K10*K31*DK21E3+K10*K21*DK31E3
         DA0E4=K21*K31*DK10E4+K10*K31*DK21E4+K10*K21*DK31E4
         DA0E5=K21*K31*DK10E5+K10*K31*DK21E5+K10*K21*DK31E5
         DA0E6=K21*K31*DK10E6+K10*K31*DK21E6+K10*K21*DK31E6

         A1 = K10*K31+K21*K31+K21*K13+K10*K21+K31*K12

         DA1E1=K21*DK10E1+K31*DK10E1+K31*DK12E1+K21*DK13E1+K10*DK21E1+
     $         K13*DK21E1+K31*DK21E1+K10*DK31E1+K12*DK31E1+K21*DK31E1
         DA1E2=K21*DK10E2+K31*DK10E2+K31*DK12E2+K21*DK13E2+K10*DK21E2+
     $         K13*DK21E2+K31*DK21E2+K10*DK31E2+K12*DK31E2+K21*DK31E2
         DA1E3=K21*DK10E3+K31*DK10E3+K31*DK12E3+K21*DK13E3+K10*DK21E3+
     $         K13*DK21E3+K31*DK21E3+K10*DK31E3+K12*DK31E3+K21*DK31E3
         DA1E4=K21*DK10E4+K31*DK10E4+K31*DK12E4+K21*DK13E4+K10*DK21E4+
     $         K13*DK21E4+K31*DK21E4+K10*DK31E4+K12*DK31E4+K21*DK31E4
         DA1E5=K21*DK10E5+K31*DK10E5+K31*DK12E5+K21*DK13E5+K10*DK21E5+
     $         K13*DK21E5+K31*DK21E5+K10*DK31E5+K12*DK31E5+K21*DK31E5
         DA1E6=K21*DK10E6+K31*DK10E6+K31*DK12E6+K21*DK13E6+K10*DK21E6+
     $         K13*DK21E6+K31*DK21E6+K10*DK31E6+K12*DK31E6+K21*DK31E6

         A2 = K10+K12+K13+K21+K31

         IF (A2.EQ.0) THEN
               WRITE(*,*) 'Small denominator found'
               IERPRD=1
               RETURN
         ENDIF

         DA2E1 = DK10E1+DK12E1+DK13E1+DK21E1+DK31E1
         DA2E2 = DK10E2+DK12E2+DK13E2+DK21E2+DK31E2
         DA2E3 = DK10E3+DK12E3+DK13E3+DK21E3+DK31E3
         DA2E4 = DK10E4+DK12E4+DK13E4+DK21E4+DK31E4
         DA2E5 = DK10E5+DK12E5+DK13E5+DK21E5+DK31E5
         DA2E6 = DK10E6+DK12E6+DK13E6+DK21E6+DK31E6

         P = A1-(A2*A2/3.0D0)

         IF (P.EQ.0) THEN
               WRITE(*,*) 'Small denominator found'
               IERPRD=1
               RETURN
         ENDIF

         DPE1=DA1E1 - 2.0D0*A2*DA2E1/3.0D0
         DPE2=DA1E2 - 2.0D0*A2*DA2E2/3.0D0
         DPE3=DA1E3 - 2.0D0*A2*DA2E3/3.0D0
         DPE4=DA1E4 - 2.0D0*A2*DA2E4/3.0D0
         DPE5=DA1E5 - 2.0D0*A2*DA2E5/3.0D0
         DPE6=DA1E6 - 2.0D0*A2*DA2E6/3.0D0

         Q = (2.0D0*A2*A2*A2/27.0D0)-(A1*A2/3.0D0)+A0

         DQE1=DA0E1-(1.0D0/3.0D0)*A2*DA1E1-(1.0D0/3.0D0)*
     $        A1*DA2E1+(2.0D0/9.0D0)*(A2*A2)*DA2E1
         DQE2=DA0E2-(1.0D0/3.0D0)*A2*DA1E2-(1.0D0/3.0D0)*
     $        A1*DA2E2+(2.0D0/9.0D0)*(A2*A2)*DA2E2
         DQE3=DA0E3-(1.0D0/3.0D0)*A2*DA1E3-(1.0D0/3.0D0)*
     $        A1*DA2E3+(2.0D0/9.0D0)*(A2*A2)*DA2E3
         DQE4=DA0E4-(1.0D0/3.0D0)*A2*DA1E4-(1.0D0/3.0D0)*
     $        A1*DA2E4+(2.0D0/9.0D0)*(A2*A2)*DA2E4
         DQE5=DA0E5-(1.0D0/3.0D0)*A2*DA1E5-(1.0D0/3.0D0)*
     $        A1*DA2E5+(2.0D0/9.0D0)*(A2*A2)*DA2E5
         DQE6=DA0E6-(1.0D0/3.0D0)*A2*DA1E6-(1.0D0/3.0D0)*
     $        A1*DA2E6+(2.0D0/9.0D0)*(A2*A2)*DA2E6

         R1 = DSQRT(-(P*P*P)/27.0D0)

         DR1E1=-(0.5D0/9.0D0)*P*P*DPE1/DSQRT(-(37.0D0/999.0D0)*P*P*P)
         DR1E2=-(0.5D0/9.0D0)*P*P*DPE2/DSQRT(-(37.0D0/999.0D0)*P*P*P)
         DR1E3=-(0.5D0/9.0D0)*P*P*DPE3/DSQRT(-(37.0D0/999.0D0)*P*P*P)
         DR1E4=-(0.5D0/9.0D0)*P*P*DPE4/DSQRT(-(37.0D0/999.0D0)*P*P*P)
         DR1E5=-(0.5D0/9.0D0)*P*P*DPE5/DSQRT(-(37.0D0/999.0D0)*P*P*P)
         DR1E6=-(0.5D0/9.0D0)*P*P*DPE6/DSQRT(-(37.0D0/999.0D0)*P*P*P)

         R2 = (-Q/2.0D0)/R1

         IF (R2.EQ.1.0D0) THEN
               WRITE(*,*) 'Small denominator found'
               IERPRD=1
               RETURN
         ENDIF

         DR2E1=-0.5D0*DQE1/R1 + 0.5D0*Q*DR1E1/(R1*R1)
         DR2E2=-0.5D0*DQE2/R1 + 0.5D0*Q*DR1E2/(R1*R1)
         DR2E3=-0.5D0*DQE3/R1 + 0.5D0*Q*DR1E3/(R1*R1)
         DR2E4=-0.5D0*DQE4/R1 + 0.5D0*Q*DR1E4/(R1*R1)
         DR2E5=-0.5D0*DQE5/R1 + 0.5D0*Q*DR1E5/(R1*R1)
         DR2E6=-0.5D0*DQE6/R1 + 0.5D0*Q*DR1E6/(R1*R1)

         IF (R2.LT.-1.0D0.OR.R2.GT.1.0D0) THEN
               WRITE(*,*) 'Illegal acos found'
               IERPRD=1
               RETURN
         ENDIF

         R3 = DACOS(R2)/3.0D0

         DR3E1=-(1.0D0/3.0D0)*DR2E1/DSQRT(1.0D0-R2*R2)
         DR3E2=-(1.0D0/3.0D0)*DR2E2/DSQRT(1.0D0-R2*R2)
         DR3E3=-(1.0D0/3.0D0)*DR2E3/DSQRT(1.0D0-R2*R2)
         DR3E4=-(1.0D0/3.0D0)*DR2E4/DSQRT(1.0D0-R2*R2)
         DR3E5=-(1.0D0/3.0D0)*DR2E5/DSQRT(1.0D0-R2*R2)
         DR3E6=-(1.0D0/3.0D0)*DR2E6/DSQRT(1.0D0-R2*R2)

         R4 = 2.0D0*DEXP(DLOG(R1)/3.0D0)

         DR4E1=(2.0D0/3.0D0)*DR1E1/(R1**(2.0D0/3.0D0))
         DR4E2=(2.0D0/3.0D0)*DR1E2/(R1**(2.0D0/3.0D0))
         DR4E3=(2.0D0/3.0D0)*DR1E3/(R1**(2.0D0/3.0D0))
         DR4E4=(2.0D0/3.0D0)*DR1E4/(R1**(2.0D0/3.0D0))
         DR4E5=(2.0D0/3.0D0)*DR1E5/(R1**(2.0D0/3.0D0))
         DR4E6=(2.0D0/3.0D0)*DR1E6/(R1**(2.0D0/3.0D0))

         L1 = -(DCOS(R3)*R4-A2/3.0D0)

         DL1E1=(1.0D0/3.0D0)*DA2E1-DCOS(R3)*DR4E1+R4*DR3E1*DSIN(R3)
         DL1E2=(1.0D0/3.0D0)*DA2E2-DCOS(R3)*DR4E2+R4*DR3E2*DSIN(R3)
         DL1E3=(1.0D0/3.0D0)*DA2E3-DCOS(R3)*DR4E3+R4*DR3E3*DSIN(R3)
         DL1E4=(1.0D0/3.0D0)*DA2E4-DCOS(R3)*DR4E4+R4*DR3E4*DSIN(R3)
         DL1E5=(1.0D0/3.0D0)*DA2E5-DCOS(R3)*DR4E5+R4*DR3E5*DSIN(R3)
         DL1E6=(1.0D0/3.0D0)*DA2E6-DCOS(R3)*DR4E6+R4*DR3E6*DSIN(R3)

         L2 = -(DCOS(R3+2.0D0*PI/3.0D0)*R4-A2/3.0D0)

         DL2E1=(1.0D0/3.0D0)*DA2E1-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E1+R4*DR3E1*DSIN(2.0D0*PI/3.0D0+R3)
         DL2E2=(1.0D0/3.0D0)*DA2E2-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E2+R4*DR3E2*DSIN(2.0D0*PI/3.0D0+R3)
         DL2E3=(1.0D0/3.0D0)*DA2E3-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E3+R4*DR3E3*DSIN(2.0D0*PI/3.0D0+R3)
         DL2E4=(1.0D0/3.0D0)*DA2E4-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E4+R4*DR3E4*DSIN(2.0D0*PI/3.0D0+R3)
         DL2E5=(1.0D0/3.0D0)*DA2E5-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E5+R4*DR3E5*DSIN(2.0D0*PI/3.0D0+R3)
         DL2E6=(1.0D0/3.0D0)*DA2E6-DCOS(2.0D0*PI/3.0D0+R3)
     $         *DR4E6+R4*DR3E6*DSIN(2.0D0*PI/3.0D0+R3)

         L3 = -(DCOS(R3+4.0D0*PI/3.0D0)*R4-A2/3.0D0)

         DL3E1=(1.0D0/3.0D0)*DA2E1-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E1+R4*DR3E1*DSIN(4.0D0*PI/3.0D0+R3)
         DL3E2=(1.0D0/3.0D0)*DA2E2-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E2+R4*DR3E2*DSIN(4.0D0*PI/3.0D0+R3)
         DL3E3=(1.0D0/3.0D0)*DA2E3-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E3+R4*DR3E3*DSIN(4.0D0*PI/3.0D0+R3)
         DL3E4=(1.0D0/3.0D0)*DA2E4-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E4+R4*DR3E4*DSIN(4.0D0*PI/3.0D0+R3)
         DL3E5=(1.0D0/3.0D0)*DA2E5-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E5+R4*DR3E5*DSIN(4.0D0*PI/3.0D0+R3)
         DL3E6=(1.0D0/3.0D0)*DA2E6-DCOS(4.0D0*PI/3.0D0+R3)
     $         *DR4E6+R4*DR3E6*DSIN(4.0D0*PI/3.0D0+R3)

C        Check for illegal values that will cause division by 0 or an overflow
         IF (CHKVAL(K21,L1).OR.CHKVAL(K31,L1).OR.
     $       CHKVAL(K21,L2).OR.CHKVAL(K31,L2).OR.
     $       CHKVAL(K21,L3).OR.CHKVAL(K31,L3).OR.
     $       CHKVAL(L1,L2).OR.CHKVAL(L1,L3).OR.CHKVAL(L2,L3)) THEN
                WRITE(*,*) 'Small denominator found'
                IERPRD=1
                RETURN
         ENDIF

         IF (L1.EQ.0.OR.L2.EQ.0.OR.L3.EQ.0) THEN
                WRITE(*,*) 'Small denominator found'
                IERPRD=1
                RETURN
         ENDIF

         C1 = 1.0D0/V1*(K21-L1)*(K31-L1)/(L1-L2)/(L1-L3)

         DC1E1 = (K31-L1)*(DK21E1-DL1E1)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E1-DL1E1)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E1-DL2E1)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E1-DL3E1)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E1/((L1-L2)*(L1-L3)*V1*V1)
         DC1E2 = (K31-L1)*(DK21E2-DL1E2)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E2-DL1E2)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E2-DL2E2)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E2-DL3E2)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E2/((L1-L2)*(L1-L3)*V1*V1)
         DC1E3 = (K31-L1)*(DK21E3-DL1E3)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E3-DL1E3)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E3-DL2E3)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E3-DL3E3)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E3/((L1-L2)*(L1-L3)*V1*V1)
         DC1E4 = (K31-L1)*(DK21E4-DL1E4)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E4-DL1E4)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E4-DL2E4)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E4-DL3E4)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E4/((L1-L2)*(L1-L3)*V1*V1)
         DC1E5 = (K31-L1)*(DK21E5-DL1E5)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E5-DL1E5)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E5-DL2E5)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E5-DL3E5)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E5/((L1-L2)*(L1-L3)*V1*V1)
         DC1E6 = (K31-L1)*(DK21E6-DL1E6)/((L1-L2)*(L1-L3)*V1)
     $           +(K21-L1)*(DK31E6-DL1E6)/((L1-L2)*(L1-L3)*V1)
     $           -(K21-L1)*(K31-L1)*(DL1E6-DL2E6)/((L1-L2)
     $           *(L1-L2)*(L1-L3)*V1)-(K21-L1)*(K31-L1)
     $           *(DL1E6-DL3E6)/((L1-L2)*(L1-L3)*(L1-L3)*V1)
     $          -(K21-L1)*(K31-L1)*DV1E6/((L1-L2)*(L1-L3)*V1*V1)

         C2 = 1.0D0/V1*(K21-L2)*(K31-L2)/(L2-L1)/(L2-L3)

         DC2E1=(K31-L2)*(DK21E1-DL2E1)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E1-DL2E1)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E1-DL1E1)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E1-DL3E1)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E1/((L2-L1)*(L2-L3)*V1*V1)
         DC2E2=(K31-L2)*(DK21E2-DL2E2)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E2-DL2E2)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E2-DL1E2)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E2-DL3E2)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E2/((L2-L1)*(L2-L3)*V1*V1)
         DC2E3=(K31-L2)*(DK21E3-DL2E3)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E3-DL2E3)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E3-DL1E3)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E3-DL3E3)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E3/((L2-L1)*(L2-L3)*V1*V1)
         DC2E4=(K31-L2)*(DK21E4-DL2E4)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E4-DL2E4)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E4-DL1E4)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E4-DL3E4)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E4/((L2-L1)*(L2-L3)*V1*V1)
         DC2E5=(K31-L2)*(DK21E5-DL2E5)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E5-DL2E5)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E5-DL1E5)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E5-DL3E5)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E5/((L2-L1)*(L2-L3)*V1*V1)
         DC2E6=(K31-L2)*(DK21E6-DL2E6)/((L2-L1)*(L2-L3)*V1)
     $         +(K21-L2)*(DK31E6-DL2E6)/((L2-L1)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*(DL2E6-DL1E6)/((L2-L1)
     $         *(L2-L1)*(L2-L3)*V1)-(K21-L2)*(K31-L2)
     $         *(DL2E6-DL3E6)/((L2-L1)*(L2-L3)*(L2-L3)*V1)
     $         -(K21-L2)*(K31-L2)*DV1E6/((L2-L1)*(L2-L3)*V1*V1)

         C3 = 1.0D0/V1*(K21-L3)*(K31-L3)/(L3-L2)/(L3-L1)

         DC3E1=(K31-L3)*(DK21E1-DL3E1)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E1-DL3E1)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E1-DL1E1)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E1-DL2E1)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E1/((L3-L1)*(L3-L2)*V1*V1)
         DC3E2=(K31-L3)*(DK21E2-DL3E2)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E2-DL3E2)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E2-DL1E2)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E2-DL2E2)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E2/((L3-L1)*(L3-L2)*V1*V1)
         DC3E3=(K31-L3)*(DK21E3-DL3E3)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E3-DL3E3)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E3-DL1E3)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E3-DL2E3)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E3/((L3-L1)*(L3-L2)*V1*V1)
         DC3E4=(K31-L3)*(DK21E4-DL3E4)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E4-DL3E4)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E4-DL1E4)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E4-DL2E4)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E4/((L3-L1)*(L3-L2)*V1*V1)
         DC3E5=(K31-L3)*(DK21E5-DL3E5)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E5-DL3E5)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E5-DL1E5)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E5-DL2E5)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E5/((L3-L1)*(L3-L2)*V1*V1)
         DC3E6=(K31-L3)*(DK21E6-DL3E6)/((L3-L1)*(L3-L2)*V1)
     $         +(K21-L3)*(DK31E6-DL3E6)/((L3-L1)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*(DL3E6-DL1E6)/((L3-L1)
     $         *(L3-L1)*(L3-L2)*V1)-(K21-L3)*(K31-L3)
     $         *(DL3E6-DL2E6)/((L3-L1)*(L3-L2)*(L3-L2)*V1)
     $         -(K21-L3)*(K31-L3)*DV1E6/((L3-L1)*(L3-L2)*V1*V1)

C    End of three compartment model..........................................
      ENDIF
      ENDIF

C Initialize state variables, including partials=============================

         S1P = 0
         DS1PE1 = 0
         DS1PE2 = 0
         DS1PE3 = 0
         DS1PE4 = 0
         DS1PE5 = 0
         DS1PE6 = 0

         S2P = 0
         DS2PE1 = 0
         DS2PE2 = 0
         DS2PE3 = 0
         DS2PE4 = 0
         DS2PE5 = 0
         DS2PE6 = 0

         S3P = 0
         DS3PE1 = 0
         DS3PE2 = 0
         DS3PE3 = 0
         DS3PE4 = 0
         DS3PE5 = 0
         DS3PE6 = 0


         CTIME = 0
         DRGLIN=DRGSTR(RECNO)
      ENDIF
C============================================================================
C============================================================================
C     Bring the model forward to the current time
C     Check if infusion must be indexed
400   IF (CTIME.GE.DRGOFF(DRGLIN).AND.
     $  (DRGON(DRGLIN+1).NE.9999.AND.DRGON(DRGLIN+1).NE.99999)) THEN
          DRGLIN = DRGLIN + 1
      ENDIF

      TON =  DRGON(DRGLIN)
      TOF =  DRGOFF(DRGLIN)
      RATE = DRGRAT(DRGLIN)

      IF (TON.LT.CTIME) THEN
         TON=CTIME
      ENDIF

      IF (TON.GT.CTIME) THEN
         TOF=TON
         TON=CTIME
         RATE=0
      ENDIF

      IF (TOF.GT.TIME) THEN
          TOF=TIME
      ENDIF

      IF (TOF.LE.CTIME) THEN
         TOF=TIME
         TON=CTIME
         RATE=0
      ENDIF

      DT  = TOF-TON

      LA1 = DEXP(-DT*L1)
      TR1 = RATE*C1*(1.0D0-LA1)/L1

      S1 = S1P*LA1+TR1
      DS1E1=(1.0D0-LA1)*RATE*DC1E1/L1-TR1*DL1E1/L1
     $      +C1*DT*RATE*DL1E1*LA1/L1
     $      -DT*S1P*DL1E1*LA1+DS1PE1*LA1
      DS1E2=(1.0D0-LA1)*RATE*DC1E2/L1-TR1*DL1E2/L1
     $      +C1*DT*RATE*DL1E2*LA1/L1
     $      -DT*S1P*DL1E2*LA1+DS1PE2*LA1
      DS1E3=(1.0D0-LA1)*RATE*DC1E3/L1-TR1*DL1E3/L1
     $      +C1*DT*RATE*DL1E3*LA1/L1
     $      -DT*S1P*DL1E3*LA1+DS1PE3*LA1
      DS1E4=(1.0D0-LA1)*RATE*DC1E4/L1-TR1*DL1E4/L1
     $      +C1*DT*RATE*DL1E4*LA1/L1
     $      -DT*S1P*DL1E4*LA1+DS1PE4*LA1
      DS1E5=(1.0D0-LA1)*RATE*DC1E5/L1-TR1*DL1E5/L1
     $      +C1*DT*RATE*DL1E5*LA1/L1
     $      -DT*S1P*DL1E5*LA1+DS1PE5*LA1
      DS1E6=(1.0D0-LA1)*RATE*DC1E6/L1-TR1*DL1E6/L1
     $      +C1*DT*RATE*DL1E6*LA1/L1
     $      -DT*S1P*DL1E6*LA1+DS1PE6*LA1

      IF (MODEL.GT.1) THEN
          LA2 = DEXP(-DT*L2)
          TR2 = RATE*C2*(1.0D0-LA2)/L2
          S2=S2P*LA2+TR2

      DS2E1=(1.0D0-LA2)*RATE*DC2E1/L2-TR2*DL2E1/L2
     $      +C2*DT*RATE*DL2E1*LA2/L2
     $      -DT*S2P*DL2E1*LA2+DS2PE1*LA2
      DS2E2=(1.0D0-LA2)*RATE*DC2E2/L2-TR2*DL2E2/L2
     $      +C2*DT*RATE*DL2E2*LA2/L2
     $      -DT*S2P*DL2E2*LA2+DS2PE2*LA2
      DS2E3=(1.0D0-LA2)*RATE*DC2E3/L2-TR2*DL2E3/L2
     $      +C2*DT*RATE*DL2E3*LA2/L2
     $      -DT*S2P*DL2E3*LA2+DS2PE3*LA2
      DS2E4=(1.0D0-LA2)*RATE*DC2E4/L2-TR2*DL2E4/L2
     $      +C2*DT*RATE*DL2E4*LA2/L2
     $      -DT*S2P*DL2E4*LA2+DS2PE4*LA2
      DS2E5=(1.0D0-LA2)*RATE*DC2E5/L2-TR2*DL2E5/L2
     $      +C2*DT*RATE*DL2E5*LA2/L2
     $      -DT*S2P*DL2E5*LA2+DS2PE5*LA2
      DS2E6=(1.0D0-LA2)*RATE*DC2E6/L2-TR2*DL2E6/L2
     $      +C2*DT*RATE*DL2E6*LA2/L2
     $      -DT*S2P*DL2E6*LA2+DS2PE6*LA2

          IF (MODEL.GT.2) THEN
              LA3 = DEXP(-DT*L3)
              TR3 = RATE*C3*(1.0D0-LA3)/L3
              S3 = S3P*LA3+TR3

      DS3E1=(1.0D0-LA3)*RATE*DC3E1/L3-TR3*DL3E1/L3
     $      +C3*DT*RATE*DL3E1*LA3/L3
     $      -DT*S3P*DL3E1*LA3+DS3PE1*LA3
      DS3E2=(1.0D0-LA3)*RATE*DC3E2/L3-TR3*DL3E2/L3
     $      +C3*DT*RATE*DL3E2*LA3/L3
     $      -DT*S3P*DL3E2*LA3+DS3PE2*LA3
      DS3E3=(1.0D0-LA3)*RATE*DC3E3/L3-TR3*DL3E3/L3
     $      +C3*DT*RATE*DL3E3*LA3/L3
     $      -DT*S3P*DL3E3*LA3+DS3PE3*LA3
      DS3E4=(1.0D0-LA3)*RATE*DC3E4/L3-TR3*DL3E4/L3
     $      +C3*DT*RATE*DL3E4*LA3/L3
     $      -DT*S3P*DL3E4*LA3+DS3PE4*LA3
      DS3E5=(1.0D0-LA3)*RATE*DC3E5/L3-TR3*DL3E5/L3
     $      +C3*DT*RATE*DL3E5*LA3/L3
     $      -DT*S3P*DL3E5*LA3+DS3PE5*LA3
      DS3E6=(1.0D0-LA3)*RATE*DC3E6/L3-TR3*DL3E6/L3
     $      +C3*DT*RATE*DL3E6*LA3/L3
     $      -DT*S3P*DL3E6*LA3+DS3PE6*LA3
          ENDIF
      ENDIF

      S1P = S1
      DS1PE1 = DS1E1
      DS1PE2 = DS1E2
      DS1PE3 = DS1E3
      DS1PE4 = DS1E4
      DS1PE5 = DS1E5
      DS1PE6 = DS1E6

      S2P = S2
      DS2PE1 = DS2E1
      DS2PE2 = DS2E2
      DS2PE3 = DS2E3
      DS2PE4 = DS2E4
      DS2PE5 = DS2E5
      DS2PE6 = DS2E6

      S3P = S3
      DS3PE1 = DS3E1
      DS3PE2 = DS3E2
      DS3PE3 = DS3E3
      DS3PE4 = DS3E4
      DS3PE5 = DS3E5
      DS3PE6 = DS3E6

      CTIME=TOF
      IF (CTIME.LT.TIME) GOTO 400
      IF (CTIME.GT.TIME) THEN
         WRITE(*,*)'CTIME GREATER THAN TIME!'
         WRITE(*,*)'CTIME=',CTIME,'TIME=',TIME
         WRITE(*,*)'ID=',ID,'RECNO=',RECNO
         STOP
      ENDIF

      TY = S1+S2+S3
      Y = TY
      H(1,1)=Y*DEXP(ETA(LSTETA))
C     write(6,*) 'y= ',y

C     Apply ETA, as indicated
      I=1
      IF (ZPETA(1).LT.7) THEN
          G(I,1)=(DS1E1+DS2E1+DS3E1)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF

      IF (ZPETA(2).LT.7) THEN
          G(I,1)=(DS1E2+DS2E2+DS3E2)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF

      IF (ZPETA(3).LT.7) THEN
          G(I,1)=(DS1E3+DS2E3+DS3E3)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF

      IF (ZPETA(4).LT.7) THEN
          G(I,1)=(DS1E4+DS2E4+DS3E4)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF

      IF (ZPETA(5).LT.7) THEN
          G(I,1)=(DS1E5+DS2E5+DS3E5)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF

      IF (ZPETA(6).LT.7) THEN
          G(I,1)=(DS1E6+DS2E6+DS3E6)
          H(1,I+1)=G(I,1)*DEXP(ETA(LSTETA))
          I=I+1
      ENDIF
C     Eta for 2 stage modeling
      G(I,1)=0
      H(1,I+1)=Y*DEXP(ETA(LSTETA))

      F=Y
      RETURN

9000  WRITE(*,*)'Error on opening ZP.NM'
      WRITE(*,*)'(probably not in current directory)'
      STOP
9001  WRITE(*,*)'Error on reading ZP.NM'
      STOP
9002  WRITE(*,*)'End of file encountered reading ZP.NM'
      STOP
9003  WRITE(*,*)'Error on opening ZPETA.NM'
      WRITE(*,*)'(probably not in current directory)'
      STOP
9004  WRITE(*,*)'Error on reading ZPETA.NM'
      STOP
9005  WRITE(*,*)'End of file encountered reading ZPETA.NM'
      STOP
9006  WRITE(*,*) 'Unable to open file COVAR.NM'
      STOP
9007  WRITE(*,*) 'Unable to open DRG file'
      WRITE(*,*) DRGFIL(RECNO)
      STOP
9008  WRITE(*,*) 'File read error in DRG file:'
      WRITE(*,*) DRGFIL(RECNO)
      STOP
9009  WRITE(*,*) 'Unexpected end of file in DRG file:'
      WRITE(*,*) DRGFIL(RECNO)
      STOP
      END

C Check for division by almost 0
      LOGICAL FUNCTION CHKVAL (A,B)
      IMPLICIT DOUBLE PRECISION (A-Z)
      IF (A.EQ.0.OR.B.EQ.0.OR.A.EQ.-B) THEN
          CHKVAL = .TRUE.
      ELSE
          IF(DABS(A-B)/(A+B).GT.0.001D0) THEN
               CHKVAL = .FALSE.
          ELSE
               CHKVAL = .TRUE.
          ENDIF
      ENDIF
      RETURN
      END

C Interpret ZP table for Ith parameter
      DOUBLE PRECISION FUNCTION PRMSET (I,THETA,ZP,COV,RECNO)
      IMPLICIT DOUBLE PRECISION (A-Z)
      REAL COV
      INTEGER I,ZP(6,11), RECNO
      DIMENSION THETA(*), COV(10,100)

      IF (ZP(I, 11).EQ.0) THEN
          TERM1 = 0.0D0
          TERM2 = 0.0D0
          TERM3 = 0.0D0
          TERM4 = 1.0D0
      ELSE
          TERM1 = 1.0D0
          TERM2 = 1.0D0
          TERM3 = 1.0D0
          TERM4 = 0.0D0
      ENDIF

C     first TERM
      IF (ZP(I, 1).GT.0) THEN
          IF (ZP(I, 2).GT.0) THEN
              TEMP = COV(ZP(I, 2), RECNO)
              IF (ZP(I, 3).GT.0) THEN
                  TEMP = TEMP**THETA(ZP(I, 3))
              ENDIF
          ELSE
              TEMP = 1
          ENDIF
          TERM1 = THETA(ZP(I, 1)) * TEMP
      ENDIF

C second term
      IF (ZP(I, 4).GT.0) THEN
          IF (ZP(I, 5).GT.0) THEN
              TEMP = COV(ZP(I,5), RECNO)
              IF (ZP(I, 6).GT.0) THEN
                  TEMP = TEMP ** THETA(ZP(I, 6))
              ENDIF
          ELSE
              TEMP = 1
          ENDIF
          TERM2 = THETA(ZP(I, 4)) * TEMP
      ENDIF

C third term
      IF (ZP(I, 7).GT.0) THEN
          IF (ZP(I, 8).GT.0) THEN
              TERM3 = THETA(ZP(I, 7)) ** COV( ZP(I,8), RECNO)
          ELSE
              TERM3 = THETA(ZP(I, 7))
          ENDIF
      ENDIF

C fourth term
      IF (ZP(I, 9).GT.0) THEN
          IF (ZP(I, 10).GT.0) THEN
              TERM4 = THETA(ZP(I, 9)) ** COV( ZP(I,10), RECNO)
          ELSE
              TERM4 = THETA(ZP(I, 9))
          ENDIF
      ENDIF

C Combine terms
      IF (ZP(I, 11).EQ.0) THEN
          PRMSET = (TERM1 + TERM2 + TERM3) * TERM4
      ELSE
          PRMSET = (TERM1 * TERM2 * TERM3) + TERM4
      ENDIF

      RETURN
      END

